"""
/*---------------------------------------------------------------------------
 *                          bootloader_py 
 *
 *  Alaeddine WESLATI, Hipercom Project-Team, INRIA Paris-Rocquencourt
 *
 *  Copyright 2013 Institut National de Recherche en Informatique et
 *  en Automatique.  All rights reserved.  Distributed only with permission.
 *---------------------------------------------------------------------------*/
"""
import sys
from optparse import OptionParser

import asyncore
import serial
import termios
import slip
import Queue
from struct import *
import binascii

class SerialReader(asyncore.file_dispatcher):
  def __init__(self, fd, map=None):
    self.q = Queue.Queue()
    self.slip = slip.slip()
    asyncore.file_dispatcher.__init__(self, fd, map)

  def writable(self):
    return not self.q.empty()

  def handle_write(self):
    data = self.q.get()
    sent = self.my_send(data)
    #print "Sent %s bytes of %s" % (sent, len(data))

  def my_send(self, data):
    """ Preprocess and send data """
    # Slip encoding
    encoded = self.slip.encode(data)

    #print "Sending %s" % repr(encoded)
    return self.send(encoded)

  def handle_read(self):
    data = self.recv(1000)
    #print " !!! " + repr(data) + " !!! "
    #print "SerialReader : READ: %d bytes" % len(data)
    self.slip.append(data)
    pktlist = self.slip.decode()

    for pkt in pktlist:
      if ("FLASH_PAGE_OK" in str(pkt)):
        print "Received Flash ACK, sending next chunck"
        self.send_next_chunk()
      else:
        print "Unknown : %s" % repr(pkt)

  def handle_expt(self):
    print "SerialReader : expt"
    pass

  def handle_close(self):
    print "SerialReader : Closing"
    self.close()
    sys.exit(0)

  def put(self, data):
    print "SerialReader : Putting %s bytes into queue" % len(data)
    self.q.put(data)
 
  def send_next_chunk(self):
    if not self.f == None:
      chunk = self.f.read(self.chunk_size)
      if len(chunk) == 0:
        self.f.close()
        # Send jump command
        self.put(pack("B", 1))
        return
      else:
        crc = binascii.crc32(chunk) & 0xffffffff
        print "Sending chunk %s, length = %s, crc = %s" % \
        (str(self.chunk_number), str(self.chunk_size), hex(crc))
        self.put(pack(">BBL", 0, self.chunk_number, crc) + chunk)
        self.chunk_number += 1

  def send_firmware(self, filename):
    self.chunk_size = 1024*2    # Page size
    self.chunk_number = 0
    self.f = open(filename, 'rb')

    self.send_next_chunk()

def openSerialDevice(options):
  try:
    print "Opening %s" % options.device
    serial_fd = serial.Serial(options.device, 
            options.baudrate, 
            serial.EIGHTBITS, 
            serial.PARITY_NONE, 
            serial.STOPBITS_ONE, 
            timeout = 0)

    # Make serial raw
    old = termios.tcgetattr(serial_fd)

    old[0] = old[0] & ~(termios.IGNBRK|termios.BRKINT|termios.PARMRK|termios.ISTRIP|termios.INLCR|termios.IGNCR|termios.ICRNL|termios.IXON)
    old[1] = old[1] & ~(termios.OPOST|termios.ONLCR)
    old[3] = old[3] & ~(termios.ECHO|termios.ECHONL|termios.ICANON|termios.ISIG|termios.IEXTEN)
    old[6][termios.VMIN] = 1
    old[6][termios.VTIME] = 0

    termios.tcsetattr(serial_fd, termios.TCSAFLUSH, old)

  except Exception as err:
    print err
    exit()
  return serial_fd

def main():
  usage = "usage: %prog [options] bin_file"
  parser = OptionParser(usage=usage)
  parser.add_option('-D', '--device',
        type='string',
        help='serial device path (default: /dev/ttyUSB1)',
        default="/dev/ttyUSB1")

  parser.add_option('-B', '--baudrate',
        type='int',
        help='Baudrate of UART (default: 500000)',
        default=500000)

  options, args = parser.parse_args()

  if len(args) != 1:
    parser.error("incorrect number of arguments")

  filename = args[0]

  serial_fd = openSerialDevice(options)
  TTYreader = SerialReader(serial_fd)

  TTYreader.send_firmware(filename)

  try:
    while asyncore.socket_map:
      asyncore.loop(timeout = 30, count = 1)

  except (KeyboardInterrupt, SystemExit):
    serial_fd.close()
    sys.exit()

if __name__ == "__main__":
  main()
