#include "platform.h"
#include "flash.h"
#include "crc.h"
#include "bootloader.h"

#include "printf.h"
/* Protocol

Byte 0 : Commmand ID : 0: write_page, 1: jump_to_user_code
  Byte 1 : Command 0,  Option : chunk id == page index 
  Byte 2 : Command 0,  Option : crc byte 0  MSB
  Byte 3 : Command 0,  Option : crc byte 1
  Byte 4 : Command 0,  Option : crc byte 2
  Byte 5 : Command 0,  Option : crc byte 3  LSB

*/

extern xSemaphoreHandle xWaitForBootloaderTriggerSemaphore;
extern xSemaphoreHandle xWaitForBootloaderTransferSemaphore;

static void jump_to_user_code();
static uint32_t write_user_code_page(uint8_t chunk_id, uint8_t* data_ptr, uint16_t len);

void serial_bridge_process_packet(uint8_t* ptr, uint16_t len)
{
  uint8_t command_id;

  command_id = ptr[0];

  if (command_id > 1)
    return;

  switch (command_id)
  {
    case 0:
      if (len > 1 + 1 + 4)
      {
        static signed portBASE_TYPE xHigherPriorityTaskWoken;
        xSemaphoreGiveFromISR(xWaitForBootloaderTriggerSemaphore, &xHigherPriorityTaskWoken);
        uint8_t chunk_id = ptr[1];
        uint32_t crc_in = ptr[5] | (((uint32_t)ptr[4]) << 8) | ((uint32_t)ptr[3] << 16) | ((uint32_t)ptr[2] << 24);
       
        uint32_t crc = write_user_code_page(chunk_id, &ptr[6], len-2-4);

        // serial ack/nack
        if (crc_in == crc)
        {
          slip_write("FLASH_PAGE_OK", 16);
        }
        else
        {
          slip_write("FLASH_PAGE_ERROR", 19);
        }
      }
      break;
    case 1:
      jump_to_user_code();
      break;
  }
}

__attribute__ ((section (".text.fastcode")))
uint32_t write_user_code_page(uint8_t chunk_id, uint8_t* data_ptr, uint16_t len)
{
  uint32_t address;
  flash_status_t status;
  uint16_t i; 
  uint32_t crc;

  // TODO : sanity checks, crc
  // Parse pkt and calculate address
  address = user_code_base_address + (((uint32_t)chunk_id) * 1024*2);

  // Disable the interrupts // TODO : use critical section functions from FreeRTOS
  __asm__("cpsid i");

  // Flash page
  flash_erase_memory_page(address);

  for (i = 0; i < len/2; i++)
  {
    uint16_t half_word;
    flash_status_t status;

    half_word = ((uint16_t)(data_ptr[i*2+1])<<8) | (uint16_t)data_ptr[i*2];
    status = flash_write_memory_half_word(address+(i*2), half_word);

//    if (status != boot_FLASH_OK)    // TODO : error handling
//      slip_write("ERROR", 6);
  }

  crc_reset();
  crc_compute(address, len/4);
  crc = crc_terminate();

  // Enable the interrupts // TODO : use critical section functions from FreeRTOS
  __asm__("cpsie i");

  return crc;
}

inline void jump_to_user_code()
{ 
  static signed portBASE_TYPE xHigherPriorityTaskWoken;
  // Jump is done in the main loop, jumping in an ISR causes ?§*%£*,
  xSemaphoreGiveFromISR(xWaitForBootloaderTransferSemaphore, &xHigherPriorityTaskWoken);
  
  //portYIELD_FROM_ISR(xHigherPriorityTaskWoken);
  //xWaitForBootloaderTriggerSemaphore = 1;
}
