#include "platform.h"
#include "slip.h"

// TODO : add contiki copyright

// chunks are one page size which is 2kB
#define RX_BUFFER_SIZE  1024*4

#define SLIP_END     0300
#define SLIP_ESC     0333
#define SLIP_ESC_END 0334
#define SLIP_ESC_ESC 0335

enum {
  STATE_TWOPACKETS = 0,	/* We have 2 packets and drop incoming data. */
  STATE_OK = 1,
  STATE_ESC = 2,
  STATE_RUBBISH = 3,
};

static uint8_t state = STATE_TWOPACKETS;
static uint16_t begin, end;
static uint8_t rxbuf[RX_BUFFER_SIZE];
static uint16_t pkt_end;

void slip_empty_buffer()
{
  begin = end = pkt_end = 0;
  state = STATE_OK;
}

slip_packet_handler_t process_packet_handler = NULL;


void slip_init(slip_packet_handler_t packet_handler)
{
  slip_empty_buffer();
  process_packet_handler = packet_handler;
}

uint8_t slip_write(const void *_ptr, int len)
{
  const uint8_t *ptr = _ptr;
  uint16_t i;
  uint8_t c;

  xputc(SLIP_END);

  for(i = 0; i < len; ++i) {
    c = *ptr++;
    if(c == SLIP_END) {
      xputc(SLIP_ESC);
      c = SLIP_ESC_END;
    } else if(c == SLIP_ESC) {
      xputc(SLIP_ESC);
      c = SLIP_ESC_ESC;
    }
    xputc(c);
  }
  xputc(SLIP_END);

  return len;
}

int8_t slip_rx_char(handler_arg_t arg, uint8_t c)
{
  switch(state) {
  case STATE_RUBBISH:
    if(c == SLIP_END) {
      state = STATE_OK;
    }
    return 0;
    
//  case STATE_TWOPACKETS:       /* Two packets are already buffered! */
//    return 0;

  case STATE_ESC:
    if(c == SLIP_ESC_END) {
      c = SLIP_END;
    } else if(c == SLIP_ESC_ESC) {
      c = SLIP_ESC;
    } else {
      state = STATE_RUBBISH;
//      SLIP_STATISTICS(slip_rubbish++);
      end = pkt_end;		/* remove rubbish */
      return 0;
    }
    state = STATE_OK;
    break;

  case STATE_OK:
    if(c == SLIP_ESC) {
      state = STATE_ESC;
      return 0;
    } else if(c == SLIP_END) {
      /*
       * We have a new packet, possibly of zero length.
       *
       * There may already be one packet buffered.
       */
      if(end != pkt_end) {	/* Non zero length. */
        if(begin == pkt_end) {	/* None buffered. */
          pkt_end = end;
        }/*else {
          state = STATE_TWOPACKETS;
          SLIP_STATISTICS(slip_twopackets++);
        }*/
        if (process_packet_handler != NULL)
          process_packet_handler(&rxbuf[begin], end-begin);
        //slip_write(&rxbuf[begin], end-begin);
        //boot_xputc('O');      // TODO
        slip_empty_buffer();

//        process_poll(&slip_process);
        return 1;
      }
      return 0;
    }
    break;
  }

  /* add_char: */
  {
    uint16_t next;
    next = end + 1;
    if(next == RX_BUFFER_SIZE) {
      next = 0;
    }
    if(next == begin) {		/* rxbuf is full */
      state = STATE_RUBBISH;
//      SLIP_STATISTICS(slip_overflow++);
      end = pkt_end;		/* remove rubbish */
      return 0;
    }
    rxbuf[end] = c;
    end = next;
  }

  return 0;
}
